@@ -193,6 +193,8 @@ const NSString *BayerProcessedDarktKey = @"procDark";
                }
             }
 
+            NSAssert((bx+dx-x >= 0) && (bx+dx-x < w) && (by-y >= 0) && (by-y < h),
+                     @"Invalid pixel position in bayer planar conversion");
             if (nPlanes == 1)
                // Convert to monochrome
                planes[0][bx+dx-x+lineW*(by-y)] = (pixel[0]+pixel[1]+pixel[2])/3.0;
