@@ -251,11 +251,13 @@ static int colorSelector(void *opaque, u_short x, u_short y)
    LynkeosImageBuffer *limg =  [self getCustomImageSampleAtX:0 Y:0 W:_width H:_height
                                                withTransform:[[NSAffineTransform transform] transformStruct]
                                                  withOffsets:nullOffsets];
-      
-   return [[[NSImage alloc] initWithCGImage:[limg getImageInRect:LynkeosMakeIntegerRect(0, 0, _width, _height)
-                                                       withBlack:vmin white:vmax gamma:g]
-                                       size:NSZeroSize]
-               autorelease];
+   CGImageRef cgImg = [limg getImageInRect:LynkeosMakeIntegerRect(0, 0, _width, _height)
+                                 withBlack:vmin white:vmax gamma:g];
+   NSImage *nsImg = [[[NSImage alloc] initWithCGImage:cgImg
+                                                 size:NSZeroSize]
+                     autorelease];
+   CGImageRelease(cgImg);
+   return nsImg;
 }
 
 - (void) setMode:(ListMode_t)mode { _mode = mode; }
