@@ -58,6 +58,105 @@ static NSCursor *myWatchCursor = nil;
 
 typedef enum { K_UP, K_DOWN } enumeration_direction_t;
 
+static NSObject *objectFromNSValue(id v)
+{
+   NSObject *n = nil;
+
+   if ([v isKindOfClass:[NSValue class]])
+   {
+      const char *fieldType = [v objCType];
+
+      if (strlen(fieldType) != 1)
+      {
+         NSLog(@"Unsupported field type : %s", fieldType );
+         return nil;
+      }
+
+      switch( *fieldType )
+      {
+         case 'c' :
+         case 'C' :
+         {
+            char c;
+            [v getValue:&c];
+            n = [NSNumber numberWithChar:c];
+         }
+            break;
+         case 's' :
+         case 'S' :
+         {
+            short s;
+            [v getValue:&s];
+            n = [NSNumber numberWithShort:s];
+         }
+            break;
+         case 'i' :
+         case 'I' :
+         {
+            int i;
+            [v getValue:&i];
+            n = [NSNumber numberWithInt:i];
+         }
+            break;
+         case 'l' :
+         case 'L' :
+         {
+            long l;
+            [v getValue:&l];
+            n = [NSNumber numberWithLong:l];
+         }
+            break;
+         case 'q' :
+         case 'Q' :
+         {
+            long long l;
+            [v getValue:&l];
+            n = [NSNumber numberWithLongLong:l];
+         }
+            break;
+         case 'f' :
+         {
+            float f;
+            [v getValue:&f];
+            n = [NSNumber numberWithFloat:f];
+         }
+            break;
+         case 'd' :
+         {
+            double d;
+            [v getValue:&d];
+            n = [NSNumber numberWithDouble:d];
+         }
+            break;
+         case 'B' :
+         {
+            BOOL b;
+            [v getValue:&b];
+            n = [NSNumber numberWithBool:b];
+         }
+            break;
+         case '*' :
+         {
+            const char *s;
+            [v getValue:&s];
+            n = [NSString stringWithCString:s encoding:NSUTF8StringEncoding];
+         }
+            break;
+         default :
+            NSLog( @"Unsupported field type %s", fieldType );
+            break;
+      }
+   }
+   else if ( [v isKindOfClass:[NSString class]])
+   {
+      n = v;
+   }
+   else
+      NSLog(@"Unsuported column data type %@", [[v class] description]);
+
+   return n;
+}
+
 /*!
  * @abstract Parameters of a processing tool
  */
@@ -129,22 +228,56 @@ typedef struct
 @implementation MyImageListWindow(Private)
 - (void) highlightOther :(enumeration_direction_t)sense skipUnselected:(BOOL)skip
 {
-   NSEnumerator *list;
    MyImageListItem *item;
-   MyImageListItem *parent;
 
-   list = [_currentList imageEnumeratorStartAt:_highlightedItem 
-                                   directSense:(sense==K_DOWN)
-                                skipUnselected:skip];
+   if (_outlineDisplay)
+   {
+      NSEnumerator *list;
+      MyImageListItem *parent;
+      list = [_currentList imageEnumeratorStartAt:_highlightedItem
+                                      directSense:(sense==K_DOWN)
+                                   skipUnselected:skip];
 
-   item = [list nextObject];
+      item = [list nextObject];
 
-   if ( item == nil )
-      return;
+      if ( item == nil )
+         return;
 
-   parent = [item getParent];
-   if ( parent != nil )
-      [_textView expandItem:parent];
+      parent = [item getParent];
+      if ( parent != nil )
+         [_textView expandItem:parent];
+   }
+   else
+   {
+      const NSUInteger lastIdx = [_sortedItems count] - 1;
+      NSUInteger idx = NSNotFound;
+
+      NSAssert(_sortedItems != nil, @"Sorted items iteration without sorted array");
+
+      item = _highlightedItem;
+      do
+      {
+         if (item != nil)
+         {
+            idx = [_sortedItems indexOfObject:item];
+            if (sense == K_DOWN && idx < lastIdx)
+               idx ++;
+            else if (sense == K_UP && idx > 0)
+               idx --;
+            else
+               idx = NSNotFound;
+         }
+         else if (sense == K_DOWN)
+            idx = 0;
+         else
+            idx =  lastIdx;
+
+         if (idx != NSNotFound && idx <= lastIdx)
+            item = [_sortedItems objectAtIndex:idx];
+         else
+            item = nil;
+      } while (skip && item != nil && [item getSelectionState] != NSOnState);
+   }
 
    [self highlightItem:item];
 }
@@ -162,6 +295,15 @@ typedef struct
       MyImageListItem *parent = [item getParent];
       if ( parent != nil )
          [_textView reloadItem:parent reloadChildren:NO];
+      // If the display is sorted, schedule a re-sorting, after a little while,
+      // in order to avoid multiple re-sortings during list processing
+      if (_dataMode == ListData && !_outlineDisplay)
+      {
+         [[self class] cancelPreviousPerformRequestsWithTarget:self
+                                                      selector:@selector(reloadData)
+                                                        object:nil];
+         [self performSelector:@selector(reloadData) withObject:nil afterDelay:0.2];
+      }
    }
 }
 
@@ -203,6 +345,10 @@ typedef struct
    _displayProgress = YES;
    _listMode = ImageMode;
    _dataMode = ListData;
+   _outlineDisplay = YES;
+   _sortedItems = nil;
+   _sortColumn = nil;
+   _sortUp = YES;
    _resultZoom = 1.0;
    _listZoom = 1.0;
 
@@ -222,6 +368,9 @@ typedef struct
 
 - (void) dealloc
 {
+   [[self class] cancelPreviousPerformRequestsWithTarget:self
+                                                selector:@selector(reloadData)
+                                                  object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [_processingViewDict release];
    if ( _processingAuthorization != NULL )
@@ -230,6 +379,8 @@ typedef struct
    [_processSubview release];
    [_marginSubview release];
    [_toolBar release];
+   if (_sortedItems != nil)
+      [_sortedItems release];
 
    [super dealloc];
 }
@@ -431,11 +582,17 @@ typedef struct
                break;
             case NSHomeFunctionKey :
                if ( nextEnabled )
-                  [self highlightItem:[_currentList firstItem]];
+               {
+                  _highlightedItem = nil;
+                  [self highlightOther:K_DOWN skipUnselected:NO];
+               }
                break;
             case NSEndFunctionKey :
                if ( nextEnabled )
-                  [self highlightItem:[_currentList lastItem]];
+               {
+                  _highlightedItem = nil;
+                  [self highlightOther:K_UP skipUnselected:NO];
+               }
                break;
             case '\r' :
             case ' ' :
@@ -688,12 +845,12 @@ typedef struct
 
 - (void) itemWasAdded:(id <LynkeosDocument>)document
 {
-   [_textView reloadData];
+   [self reloadData];
 }
 
 - (void) itemWasRemoved:(id <LynkeosDocument>)document
 {
-   [_textView reloadData];
+   [self reloadData];
    // Force update of the hilighted item in case it was deleted
    // Because outline view does not notify (the selection is at same line)
    [self outlineViewSelectionDidChange:
@@ -712,7 +869,7 @@ typedef struct
 
    _currentList = [(MyDocument*)document currentList];
    // Redisplay the outline view
-   [_textView reloadData];
+   [self reloadData];
 
    if ( _dataMode == ResultData )
       // Force selection on first and only line
@@ -733,7 +890,7 @@ typedef struct
       _dataMode = mode;
       [_dataModeRadio selectCellWithTag:_dataMode];
       // Redisplay the outline view
-      [_textView reloadData];
+      [self reloadData];
       // Reset the bounds of the image view, which were adapted to the "other" mode
       [_imageView resetBounds];
 
@@ -1369,6 +1526,7 @@ typedef struct
    }
 }
 
+#pragma mark Other
 - (void) delete:(id)sender
 {
    NSInteger sel = [_textView selectedRow];
@@ -1417,6 +1575,86 @@ typedef struct
 
 - (void) reloadData
 {
+   if (_dataMode == ListData && !_outlineDisplay)
+   { // Re-sort before reloading
+      // Find the sort column description
+      LynkeosColumnDescription *desc = [_columnsDescriptor objectForKey:_sortColumn];
+      NSMutableArray *unsortedItems = [NSMutableArray array];
+      NSEnumerator *listEnum = [_currentList imageEnumerator];
+      MyImageListItem *item;
+
+      while ((item = [listEnum nextObject]) != nil)
+         [unsortedItems addObject:item];
+
+      if (_sortedItems != nil)
+         [_sortedItems release];
+      if (desc != nil)
+      {
+         _sortedItems = [unsortedItems sortedArrayUsingComparator: ^(id obj1, id obj2) {
+
+            NSObject <LynkeosProcessingParameter> *param1 =
+            [obj1 getProcessingParameterWithRef:desc->_parameterReference
+                                  forProcessing:desc->_processingRef];
+            NSObject <LynkeosProcessingParameter> *param2 =
+            [obj2 getProcessingParameterWithRef:desc->_parameterReference
+                                  forProcessing:desc->_processingRef];
+            NSObject *field1 = nil, *field2 = nil;
+            if (param1 != nil)
+               field1 = objectFromNSValue([param1 valueForKey:desc->_fieldName]);
+            if (param2 != nil)
+               field2 = objectFromNSValue([param2 valueForKey:desc->_fieldName]);
+
+            if (field1 == nil)
+            {
+               if (field2 == nil)
+                  return NSOrderedSame;
+               else if (_sortUp)
+                  return NSOrderedAscending;
+               else
+                  return NSOrderedDescending;
+            }
+            else if (field2 == nil)
+            {
+               if (_sortUp)
+                  return NSOrderedDescending;
+               else
+                  return NSOrderedAscending;
+            }
+            else
+            {
+               if ([field1 isKindOfClass:[NSNumber class]])
+               {
+                  NSNumber *num1 = (NSNumber*)field1;
+                  NSNumber *num2 = (NSNumber*)field2;
+                  if (_sortUp)
+                     return [num1 compare:num2];
+                  else
+                     return [num2 compare:num1];
+               }
+               else if ([field1 isKindOfClass:[NSString class]])
+               {
+                  NSString *str1 = (NSString*)field1;
+                  NSString *str2 = (NSString*)field2;
+                  if (_sortUp)
+                     return [str1 localizedStandardCompare:str2];
+                  else
+                     return [str2 localizedStandardCompare:str1];
+               }
+               else
+               {
+                  NSLog(@"Unsupported type for sorting %@", [[field1 class] description]);
+                  return NSOrderedSame;
+               }
+            }
+         }];
+      }
+      else
+         // Best effort...
+         _sortedItems = unsortedItems;
+
+      [_sortedItems retain];
+   }
+
    [_textView reloadData];
 }
 
